@@ -1,10 +1,10 @@
 use crate::frame_info::{GameInput, BLANK_INPUT};
 use crate::network::compression::{decode, encode};
+use crate::network::non_blocking_socket::NonBlockingSocket;
 use crate::network::udp_msg::{
     ConnectionStatus, Input, InputAck, MessageBody, MessageHeader, QualityReply, QualityReport,
     SyncReply, SyncRequest, UdpMessage,
 };
-use crate::network::udp_socket::NonBlockingSocket;
 use crate::sessions::p2p_session::{
     Event, DEFAULT_DISCONNECT_NOTIFY_START, DEFAULT_DISCONNECT_TIMEOUT, DEFAULT_FPS,
 };
@@ -188,7 +188,7 @@ impl UdpProtocol {
             return;
         }
         // Estimate which frame the other client is on by looking at the last frame they gave us plus some delta for the packet roundtrip time.
-        let ping = i32::try_from(self.round_trip_time).expect("Ping is higher than i32::MAX");
+        let ping = i32::try_from(self.round_trip_time / 2).expect("Ping is higher than i32::MAX");
         let remote_frame = self.last_received_input.frame + ((ping * self.fps as i32) / 1000);
         // Our frame "advantage" is how many frames behind the remote client we are. (It's an advantage because they will have to predict more often)
         self.local_frame_advantage = remote_frame - local_frame;
@@ -213,8 +213,12 @@ impl UdpProtocol {
         }
 
         let now = millis_since_epoch();
-        let total_bytes_sent = self.bytes_sent + (self.packets_sent * UDP_HEADER_SIZE);
         let seconds = (now - self.stats_start_time) / 1000;
+        if seconds == 0 {
+            return None;
+        }
+
+        let total_bytes_sent = self.bytes_sent + (self.packets_sent * UDP_HEADER_SIZE);
         let bps = total_bytes_sent / seconds as usize;
         //let upd_overhead = (self.packets_sent * UDP_HEADER_SIZE) / self.bytes_sent;
 
@@ -263,9 +267,8 @@ impl UdpProtocol {
         self.send_sync_request();
     }
 
-    pub(crate) fn recommend_frame_delay(&self, require_idle_input: bool) -> u32 {
-        self.time_sync_layer
-            .recommend_frame_delay(require_idle_input)
+    pub(crate) fn average_frame_advantage(&self) -> i32 {
+        self.time_sync_layer.average_frame_advantage()
     }
 
     pub(crate) fn poll(&mut self, connect_status: &[ConnectionStatus]) -> Drain<Event> {
@@ -338,7 +341,7 @@ impl UdpProtocol {
      *  SENDING MESSAGES
      */
 
-    pub(crate) fn send_all_messages(&mut self, socket: &NonBlockingSocket) {
+    pub(crate) fn send_all_messages(&mut self, socket: &mut Box<dyn NonBlockingSocket>) {
         if self.state == ProtocolState::Shutdown {
             self.send_queue.drain(..);
             return;
@@ -356,7 +359,7 @@ impl UdpProtocol {
 
         // register the input and advantages in the time sync layer
         self.time_sync_layer.advance_frame(
-            input,
+            input.frame,
             self.local_frame_advantage,
             self.remote_frame_advantage,
         );
