@@ -1,23 +1,25 @@
 use crate::error::GGRSError;
 use crate::frame_info::GameInput;
 use crate::network::network_stats::NetworkStats;
+use crate::network::non_blocking_socket::{NonBlockingSocket, UdpNonBlockingSocket};
 use crate::network::udp_msg::ConnectionStatus;
 use crate::network::udp_protocol::UdpProtocol;
-use crate::network::udp_socket::NonBlockingSocket;
 use crate::sync_layer::SyncLayer;
 use crate::{
-    Frame, GGRSEvent, GGRSRequest, PlayerHandle, PlayerType, SessionState, MAX_PREDICTION_FRAMES,
-    NULL_FRAME,
+    Frame, GGRSEvent, GGRSRequest, PlayerHandle, PlayerType, SessionState, MAX_INPUT_BYTES,
+    MAX_PLAYERS, MAX_PREDICTION_FRAMES, NULL_FRAME,
 };
 
 use std::collections::vec_deque::Drain;
 use std::collections::HashMap;
 use std::collections::VecDeque;
+use std::convert::TryInto;
 use std::net::{IpAddr, Ipv4Addr, SocketAddr};
 use std::time::Duration;
 
 /// The minimum amounts of frames between sleeps to compensate being ahead of other players
-const RECOMMENDATION_INTERVAL: Frame = 40;
+const RECOMMENDATION_INTERVAL: Frame = 60;
+const MIN_RECOMMENDATION: u32 = 3;
 const MAX_EVENT_QUEUE_SIZE: usize = 100;
 const DEFAULT_SAVE_MODE: bool = false;
 pub(crate) const DEFAULT_DISCONNECT_TIMEOUT: Duration = Duration::from_millis(2000);
@@ -118,8 +120,8 @@ pub struct P2PSession {
     /// Internal State of the Session.
     state: SessionState,
 
-    /// The `P2PSession` uses this UDP socket to send and receive all messages for remote players.
-    socket: NonBlockingSocket,
+    /// The `P2PSession` uses this socket to send and receive all messages for remote players.
+    socket: Box<dyn NonBlockingSocket>,
     /// A map of player handle to a player struct that handles receiving and sending messages for remote players, remote spectators and register local players.
     players: HashMap<PlayerHandle, Player>,
     /// This struct contains information about remote players, like connection status and the frame of last received input.
@@ -129,27 +131,79 @@ pub struct P2PSession {
     next_spectator_frame: Frame,
     /// The soonest frame on which the session can send a `GGRSEvent::WaitRecommendation` again.
     next_recommended_sleep: Frame,
+    /// How many frames we estimate we are ahead of every remote client
+    frames_ahead: i32,
 
     ///Contains all events to be forwarded to the user.
     event_queue: VecDeque<GGRSEvent>,
 }
 
 impl P2PSession {
-    pub(crate) fn new(
+    /// Creates a new `P2PSession` for players who participate on the game input. After creating the session, add local and remote players,
+    /// set input delay for local players and then start the session.
+    /// # Example
+    ///
+    /// ```
+    /// # use ggrs::{GGRSError, P2PSession};
+    /// # fn main() -> Result<(), GGRSError> {
+    /// let local_port: u16 = 7777;
+    /// let num_players : u32 = 2;
+    /// let input_size : usize = std::mem::size_of::<u32>();
+    /// let mut session = P2PSession::new(num_players, input_size, local_port)?;
+    /// # Ok(())
+    /// # }
+    /// ```
+    ///
+    /// The created session will use the default socket type (currently UDP).
+    ///
+    /// # Errors
+    /// - Will return a `InvalidRequest` if the number of players is higher than the allowed maximum (see `MAX_PLAYERS`).
+    /// - Will return a `InvalidRequest` if `input_size` is higher than the allowed maximum (see `MAX_INPUT_BYTES`).
+    /// - Will return `SocketCreationFailed` if the socket could not be created.
+    pub fn new(num_players: u32, input_size: usize, local_port: u16) -> Result<Self, GGRSError> {
+        // udp nonblocking socket creation
+        let addr = SocketAddr::new(IpAddr::V4(Ipv4Addr::UNSPECIFIED), local_port); //TODO: IpV6?
+        let socket =
+            Box::new(UdpNonBlockingSocket::new(addr).map_err(|_| GGRSError::SocketCreationFailed)?);
+        Self::new_impl(num_players, input_size, socket)
+    }
+
+    /// Creates a new `P2PSession` for players who participate on the game input. After creating the session, add local and remote players,
+    /// set input delay for local players and then start the session. The session will use the provided socket.
+    ///
+    /// # Errors
+    /// - Will return a `InvalidRequest` if the number of players is higher than the allowed maximum (see `MAX_PLAYERS`).
+    /// - Will return a `InvalidRequest` if `input_size` is higher than the allowed maximum (see `MAX_INPUT_BYTES`).
+    pub fn new_with_socket(
         num_players: u32,
         input_size: usize,
-        port: u16,
-    ) -> Result<Self, std::io::Error> {
+        socket: impl NonBlockingSocket + 'static,
+    ) -> Result<Self, GGRSError> {
+        Self::new_impl(num_players, input_size, Box::new(socket))
+    }
+
+    fn new_impl(
+        num_players: u32,
+        input_size: usize,
+        socket: Box<dyn NonBlockingSocket>,
+    ) -> Result<Self, GGRSError> {
+        if num_players > MAX_PLAYERS {
+            return Err(GGRSError::InvalidRequest {
+                info: "Too many players.".to_owned(),
+            });
+        }
+        if input_size > MAX_INPUT_BYTES {
+            return Err(GGRSError::InvalidRequest {
+                info: "Input size too big.".to_owned(),
+            });
+        }
+
         // local connection status
         let mut local_connect_status = Vec::new();
         for _ in 0..num_players {
             local_connect_status.push(ConnectionStatus::default());
         }
 
-        // udp nonblocking socket creation
-        let addr = SocketAddr::new(IpAddr::V4(Ipv4Addr::new(0, 0, 0, 0)), port); //TODO: IpV6?
-        let socket = NonBlockingSocket::new(addr)?;
-
         Ok(Self {
             state: SessionState::Initializing,
             num_players,
@@ -160,6 +214,7 @@ impl P2PSession {
             local_connect_status,
             next_recommended_sleep: 0,
             next_spectator_frame: 0,
+            frames_ahead: 0,
             sync_layer: SyncLayer::new(num_players, input_size),
             disconnect_timeout: DEFAULT_DISCONNECT_TIMEOUT,
             disconnect_notify_start: DEFAULT_DISCONNECT_NOTIFY_START,
@@ -335,7 +390,7 @@ impl P2PSession {
             );
         }
 
-        // send confirmed inputs to remotes
+        // send confirmed inputs to spectators
         self.send_confirmed_inputs_to_spectators(confirmed_frame);
 
         // set the last confirmed frame and discard all saved inputs before that frame
@@ -343,14 +398,17 @@ impl P2PSession {
             .set_last_confirmed_frame(confirmed_frame, self.sparse_saving);
 
         // check time sync between clients and send wait recommendation, if appropriate
-        if self.sync_layer.current_frame() > self.next_recommended_sleep {
-            let skip_frames = self.max_delay_recommendation(true);
-            if skip_frames > 0 {
-                self.next_recommended_sleep =
-                    self.sync_layer.current_frame() + RECOMMENDATION_INTERVAL;
-                self.event_queue
-                    .push_back(GGRSEvent::WaitRecommendation { skip_frames });
-            }
+        self.frames_ahead = self.max_frame_advantage();
+        if self.sync_layer.current_frame() > self.next_recommended_sleep
+            && self.frames_ahead >= MIN_RECOMMENDATION as i32
+        {
+            self.next_recommended_sleep = self.sync_layer.current_frame() + RECOMMENDATION_INTERVAL;
+            self.event_queue.push_back(GGRSEvent::WaitRecommendation {
+                skip_frames: self
+                    .frames_ahead
+                    .try_into()
+                    .expect("frames ahead is negative despite being positive."),
+            });
         }
 
         //create an input struct for current frame
@@ -376,11 +434,11 @@ impl P2PSession {
             {
                 // send the input directly
                 endpoint.send_input(game_input, &self.local_connect_status);
-                endpoint.send_all_messages(&self.socket);
+                endpoint.send_all_messages(&mut self.socket);
             }
         }
 
-        // without sparse saving, always save the current frame
+        // without sparse saving, always save the current frame after correcting and rollbacking
         if !self.sparse_saving {
             requests.push(self.sync_layer.save_current_state());
         }
@@ -394,7 +452,7 @@ impl P2PSession {
             assert!(input.frame == NULL_FRAME || input.frame == self.sync_layer.current_frame());
         }
 
-        // advance the frame
+        // advance the frame count
         self.sync_layer.advance_frame();
         requests.push(GGRSRequest::AdvanceFrame { inputs });
 
@@ -454,7 +512,7 @@ impl P2PSession {
             .values_mut()
             .filter_map(Player::as_endpoint_mut)
         {
-            endpoint.send_all_messages(&self.socket);
+            endpoint.send_all_messages(&mut self.socket);
         }
     }
 
@@ -604,6 +662,11 @@ impl P2PSession {
         self.input_size
     }
 
+    /// Returns the number of frames this session is estimated to be ahead of other sessions
+    pub const fn frames_ahead(&self) -> i32 {
+        self.frames_ahead
+    }
+
     fn add_local_player(&mut self, player_handle: PlayerHandle) -> Result<PlayerHandle, GGRSError> {
         // check if valid player
         if player_handle >= self.num_players as PlayerHandle {
@@ -792,12 +855,12 @@ impl P2PSession {
     }
 
     /// For each spectator, send all confirmed input up until the minimum confirmed frame.
-    fn send_confirmed_inputs_to_spectators(&mut self, min_confirmed_frame: Frame) {
+    fn send_confirmed_inputs_to_spectators(&mut self, confirmed_frame: Frame) {
         if self.num_spectators() == 0 {
             return;
         }
 
-        while self.next_spectator_frame <= min_confirmed_frame {
+        while self.next_spectator_frame <= confirmed_frame {
             let inputs = self
                 .sync_layer
                 .confirmed_inputs(self.next_spectator_frame, &self.local_connect_status);
@@ -884,9 +947,9 @@ impl P2PSession {
         }
     }
 
-    /// Gather delay recommendations from each remote client and return the maximum.
-    fn max_delay_recommendation(&self, require_idle_input: bool) -> u32 {
-        let mut interval = 0;
+    /// Gather average frame advantage from each remote client and return the maximum.
+    fn max_frame_advantage(&self) -> i32 {
+        let mut interval = i32::MIN;
         for (player_handle, endpoint) in self
             .players
             .values()
@@ -894,10 +957,16 @@ impl P2PSession {
             .enumerate()
         {
             if !self.local_connect_status[player_handle].disconnected {
-                interval =
-                    std::cmp::max(interval, endpoint.recommend_frame_delay(require_idle_input));
+                // TODO: is this still what we want for >2 players?
+                interval = std::cmp::max(interval, endpoint.average_frame_advantage());
             }
         }
+
+        // if no remote player is connected
+        if interval == i32::MIN {
+            interval = 0;
+        }
+
         interval
     }
 
