@@ -6,9 +6,12 @@ use std::{
 use crate::{
     frame_info::BLANK_INPUT,
     network::{
-        udp_msg::ConnectionStatus, udp_protocol::UdpProtocol, udp_socket::NonBlockingSocket,
+        non_blocking_socket::{NonBlockingSocket, UdpNonBlockingSocket},
+        udp_msg::ConnectionStatus,
+        udp_protocol::UdpProtocol,
     },
-    Frame, GGRSError, GGRSEvent, GGRSRequest, GameInput, NetworkStats, SessionState, NULL_FRAME,
+    Frame, GGRSError, GGRSEvent, GGRSRequest, GameInput, NetworkStats, SessionState,
+    MAX_INPUT_BYTES, MAX_PLAYERS, NULL_FRAME,
 };
 
 use super::p2p_session::Event;
@@ -33,7 +36,7 @@ pub struct P2PSpectatorSession {
     input_size: usize,
     inputs: [GameInput; SPECTATOR_BUFFER_SIZE],
     host_connect_status: Vec<ConnectionStatus>,
-    socket: NonBlockingSocket,
+    socket: Box<dyn NonBlockingSocket>,
     host: UdpProtocol,
     event_queue: VecDeque<GGRSEvent>,
     current_frame: Frame,
@@ -43,15 +46,74 @@ pub struct P2PSpectatorSession {
 }
 
 impl P2PSpectatorSession {
-    pub(crate) fn new(
+    /// Creates a new `P2PSpectatorSession` for a spectator.
+    /// The session will receive inputs from all players from the given host directly.
+    /// # Example
+    ///
+    /// ```
+    /// # use std::net::SocketAddr;
+    /// # use ggrs::P2PSpectatorSession;
+    /// # fn main() -> Result<(), Box<dyn std::error::Error>> {
+    /// let local_port: u16 = 7777;
+    /// let num_players : u32 = 2;
+    /// let input_size : usize = std::mem::size_of::<u32>();
+    /// let host_addr: SocketAddr = "127.0.0.1:8888".parse()?;
+    /// let mut session = P2PSpectatorSession::new(num_players, input_size, local_port, host_addr)?;
+    /// # Ok(())
+    /// # }
+    /// ```
+    ///
+    /// The created session will use the default socket type (currently UDP).
+    ///
+    /// # Errors
+    /// - Will return a `InvalidRequest` if the number of players is higher than the allowed maximum (see `MAX_PLAYERS`).
+    /// - Will return a `InvalidRequest` if `input_size` is higher than the allowed maximum (see `MAX_INPUT_BYTES`).
+    /// - Will return `SocketCreationFailed` if the socket could not be created.
+    pub fn new(
         num_players: u32,
         input_size: usize,
         local_port: u16,
         host_addr: SocketAddr,
-    ) -> Result<Self, std::io::Error> {
+    ) -> Result<Self, GGRSError> {
         // udp nonblocking socket creation
-        let addr = SocketAddr::new(IpAddr::V4(Ipv4Addr::new(0, 0, 0, 0)), local_port); //TODO: IpV6?
-        let socket = NonBlockingSocket::new(addr)?;
+        let addr = SocketAddr::new(IpAddr::V4(Ipv4Addr::UNSPECIFIED), local_port); //TODO: IpV6?
+        let socket =
+            Box::new(UdpNonBlockingSocket::new(addr).map_err(|_| GGRSError::SocketCreationFailed)?);
+        Self::new_impl(num_players, input_size, socket, host_addr)
+    }
+
+    /// Creates a new `P2PSpectatorSession` for a spectator.
+    /// The session will receive inputs from all players from the given host directly.
+    /// The session will use the provided socket.
+    ///
+    /// # Errors
+    /// - Will return a `InvalidRequest` if the number of players is higher than the allowed maximum (see `MAX_PLAYERS`).
+    /// - Will return a `InvalidRequest` if `input_size` is higher than the allowed maximum (see `MAX_INPUT_BYTES`).
+    pub fn new_with_socket(
+        num_players: u32,
+        input_size: usize,
+        socket: impl NonBlockingSocket + 'static,
+        host_addr: SocketAddr,
+    ) -> Result<Self, GGRSError> {
+        Self::new_impl(num_players, input_size, Box::new(socket), host_addr)
+    }
+
+    fn new_impl(
+        num_players: u32,
+        input_size: usize,
+        socket: Box<dyn NonBlockingSocket>,
+        host_addr: SocketAddr,
+    ) -> Result<Self, GGRSError> {
+        if num_players > MAX_PLAYERS {
+            return Err(GGRSError::InvalidRequest {
+                info: "Too many players.".to_owned(),
+            });
+        }
+        if input_size > MAX_INPUT_BYTES {
+            return Err(GGRSError::InvalidRequest {
+                info: "Input size too big.".to_owned(),
+            });
+        }
 
         // host connection status
         let mut host_connect_status = Vec::new();
@@ -220,7 +282,7 @@ impl P2PSpectatorSession {
         }
 
         // send out all pending UDP messages
-        self.host.send_all_messages(&self.socket);
+        self.host.send_all_messages(&mut self.socket);
     }
 
     /// Returns the number of players this session was constructed with.
